@@ -69,8 +69,11 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 			Gets or sets variables on the master channel
 		</synopsis>
 		<description>
-			<para>Allows access to the channel which created the current channel, if any.  If the channel is already
-			a master channel, then accesses local channel variables.</para>
+			<para>Allows access to the oldest channel associated with the current
+			channel if it still exists.  If the channel is the master channel or
+			the master channel no longer exists then access local channel variables
+			instead.  In other words, the master channel is the channel identified by
+			the channel's linkedid.</para>
 		</description>
 	</function>
 	<function name="CHANNEL" language="en_US">
@@ -201,10 +204,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 					<enum name="videonativeformat">
 						<para>R/O format used natively for video</para>
 					</enum>
-					<enum name="trace">
-						<para>R/W whether or not context tracing is enabled, only available
-						<emphasis>if CHANNEL_TRACE is defined</emphasis>.</para>
-					</enum>
 					<enum name="hangupsource">
 						<para>R/W returns the channel responsible for hangup.</para>
 					</enum>
@@ -324,10 +323,6 @@ static int func_channel_read(struct ast_channel *chan, const char *function,
 		locked_copy_string(chan, buf, ast_format_get_name(ast_channel_readformat(chan)), len);
 	} else if (!strcasecmp(data, "audiowriteformat")) {
 		locked_copy_string(chan, buf, ast_format_get_name(ast_channel_writeformat(chan)), len);
-#ifdef CHANNEL_TRACE
-	} else if (!strcasecmp(data, "trace")) {
-		locked_copy_string(chan, buf, ast_channel_trace_is_enabled(chan) ? "1" : "0", len);
-#endif
 	} else if (!strcasecmp(data, "tonezone") && ast_channel_zone(chan)) {
 		locked_copy_string(chan, buf, ast_channel_zone(chan)->country, len);
 	} else if (!strcasecmp(data, "dtmf_features")) {
@@ -492,38 +487,23 @@ static int func_channel_write_real(struct ast_channel *chan, const char *functio
 			ast_bridge_set_after_go_on(chan, ast_channel_context(chan), ast_channel_exten(chan), ast_channel_priority(chan), value);
 		}
 	} else if (!strcasecmp(data, "amaflags")) {
-		ast_channel_lock(chan);
+		int amaflags;
+
 		if (isdigit(*value)) {
-			int amaflags;
-			sscanf(value, "%30d", &amaflags);
-			ast_channel_amaflags_set(chan, amaflags);
-		} else if (!strcasecmp(value,"OMIT")){
-			ast_channel_amaflags_set(chan, 1);
-		} else if (!strcasecmp(value,"BILLING")){
-			ast_channel_amaflags_set(chan, 2);
-		} else if (!strcasecmp(value,"DOCUMENTATION")){
-			ast_channel_amaflags_set(chan, 3);
+			if (sscanf(value, "%30d", &amaflags) != 1) {
+				amaflags = AST_AMA_NONE;
+			}
+		} else {
+			amaflags = ast_channel_string2amaflag(value);
 		}
+		ast_channel_lock(chan);
+		ast_channel_amaflags_set(chan, amaflags);
 		ast_channel_unlock(chan);
 	} else if (!strcasecmp(data, "peeraccount"))
 		locked_string_field_set(chan, peeraccount, value);
 	else if (!strcasecmp(data, "hangupsource"))
 		/* XXX - should we be forcing this here? */
 		ast_set_hangupsource(chan, value, 0);
-#ifdef CHANNEL_TRACE
-	else if (!strcasecmp(data, "trace")) {
-		ast_channel_lock(chan);
-		if (ast_true(value))
-			ret = ast_channel_trace_enable(chan);
-		else if (ast_false(value))
-			ret = ast_channel_trace_disable(chan);
-		else {
-			ret = -1;
-			ast_log(LOG_WARNING, "Invalid value for CHANNEL(trace).\n");
-		}
-		ast_channel_unlock(chan);
-	}
-#endif
 	else if (!strcasecmp(data, "tonezone")) {
 		struct ast_tone_zone *new_zone;
 		if (!(new_zone = ast_get_indication_zone(value))) {
