@@ -44,6 +44,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include "asterisk/netsock2.h"
 #include "asterisk/vector.h"
 #include "asterisk/lock.h"
+#include "asterisk/res_pjproject.h"
 
 #define HISTORY_INITIAL_SIZE 256
 
@@ -707,10 +708,13 @@ static pj_status_t history_on_tx_msg(pjsip_tx_data *tdata)
 	pj_sockaddr_cp(&entry->dst, &tdata->tp_info.dst_addr);
 
 	ast_mutex_lock(&history_lock);
-	AST_VECTOR_APPEND(&vector_history, entry);
+	if (AST_VECTOR_APPEND(&vector_history, entry)) {
+		ao2_ref(entry, -1);
+		entry = NULL;
+	}
 	ast_mutex_unlock(&history_lock);
 
-	if (log_level != -1) {
+	if (log_level != -1 && entry) {
 		char line[256];
 
 		sprint_list_entry(entry, line, sizeof(line));
@@ -747,10 +751,13 @@ static pj_bool_t history_on_rx_msg(pjsip_rx_data *rdata)
 	}
 
 	ast_mutex_lock(&history_lock);
-	AST_VECTOR_APPEND(&vector_history, entry);
+	if (AST_VECTOR_APPEND(&vector_history, entry)) {
+		ao2_ref(entry, -1);
+		entry = NULL;
+	}
 	ast_mutex_unlock(&history_lock);
 
-	if (log_level != -1) {
+	if (log_level != -1 && entry) {
 		char line[256];
 
 		sprint_list_entry(entry, line, sizeof(line));
@@ -961,7 +968,9 @@ static int evaluate_history_entry(struct pjsip_history_entry *entry, struct expr
 
 		/* If this is not an operator, push it to the stack */
 		if (!it_queue->op) {
-			AST_VECTOR_APPEND(&stack, it_queue);
+			if (AST_VECTOR_APPEND(&stack, it_queue)) {
+				goto error;
+			}
 			continue;
 		}
 
@@ -1037,7 +1046,11 @@ static int evaluate_history_entry(struct pjsip_history_entry *entry, struct expr
 		if (!result) {
 			goto error;
 		}
-		AST_VECTOR_APPEND(&stack, result);
+		if (AST_VECTOR_APPEND(&stack, result)) {
+			expression_token_free(result);
+
+			goto error;
+		}
 	}
 
 	/*
@@ -1058,6 +1071,7 @@ static int evaluate_history_entry(struct pjsip_history_entry *entry, struct expr
 	}
 	result = final->result;
 	ast_free(final);
+	AST_VECTOR_FREE(&stack);
 
 	return result;
 
@@ -1100,6 +1114,7 @@ static struct vector_history_t *filter_history(struct ast_cli_args *a)
 
 	queue = build_expression_queue(a);
 	if (!queue) {
+		AST_VECTOR_PTR_FREE(output);
 		return NULL;
 	}
 
@@ -1120,7 +1135,10 @@ static struct vector_history_t *filter_history(struct ast_cli_args *a)
 		} else if (!res) {
 			continue;
 		} else {
-			AST_VECTOR_APPEND(output, ao2_bump(entry));
+			ao2_bump(entry);
+			if (AST_VECTOR_APPEND(output, entry)) {
+				ao2_cleanup(entry);
+			}
 		}
 	}
 	ast_mutex_unlock(&history_lock);
@@ -1266,7 +1284,7 @@ static char *pjsip_show_history(struct ast_cli_entry *e, int cmd, struct ast_cli
 		}
 		entry = ao2_bump(AST_VECTOR_GET(vec, 0));
 		if (vec == &vector_history) {
-			ast_mutex_lock(&history_lock);
+			ast_mutex_unlock(&history_lock);
 		}
 	}
 
@@ -1357,7 +1375,7 @@ static int load_module(void)
 		ast_log(LOG_WARNING, "Unable to register history log level\n");
 	}
 
-	pj_caching_pool_init(&cachingpool, &pj_pool_factory_default_policy, 0);
+	ast_pjproject_caching_pool_init(&cachingpool, &pj_pool_factory_default_policy, 0);
 
 	AST_VECTOR_INIT(&vector_history, HISTORY_INITIAL_SIZE);
 
@@ -1372,10 +1390,10 @@ static int unload_module(void)
 	ast_cli_unregister_multiple(cli_pjsip, ARRAY_LEN(cli_pjsip));
 	ast_sip_unregister_service(&logging_module);
 
-	ast_sip_push_task_synchronous(NULL, clear_history_entries, NULL);
+	ast_sip_push_task_wait_servant(NULL, clear_history_entries, NULL);
 	AST_VECTOR_FREE(&vector_history);
 
-	pj_caching_pool_destroy(&cachingpool);
+	ast_pjproject_caching_pool_destroy(&cachingpool);
 
 	if (log_level != -1) {
 		ast_logger_unregister_level("PJSIP_HISTORY");
